@@ -296,7 +296,11 @@ rounded_polygon (cairo_t * cr,
 
 static void
 draw_polygon (MatekbdKeyboardDrawingRenderContext * context,
+#if GTK_CHECK_VERSION (3, 0, 0)
+	      GdkRGBA * fill_color,
+#else
 	      GdkColor * fill_color,
+#endif
 	      gint xkb_x,
 	      gint xkb_y, XkbPointRec * xkb_points, guint num_points,
 	      gdouble radius)
@@ -308,11 +312,15 @@ draw_polygon (MatekbdKeyboardDrawingRenderContext * context,
 	if (fill_color) {
 		filled = TRUE;
 	} else {
-		fill_color = context->dark_color;
+		fill_color = &context->dark_color;
 		filled = FALSE;
 	}
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_cairo_set_source_rgba (context->cr, fill_color);
+#else
 	gdk_cairo_set_source_color (context->cr, fill_color);
+#endif
 
 	points = g_new (GdkPoint, num_points);
 
@@ -368,12 +376,20 @@ curve_rectangle (cairo_t * cr,
 static void
 draw_curve_rectangle (cairo_t * cr,
 		      gboolean filled,
+#if GTK_CHECK_VERSION (3, 0, 0)
+		      GdkRGBA * fill_color,
+#else
 		      GdkColor * fill_color,
+#endif
 		      gint x, gint y, gint width, gint height, gint radius)
 {
 	curve_rectangle (cr, x, y, width, height, radius);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_cairo_set_source_rgba (cr, fill_color);
+#else
 	gdk_cairo_set_source_color (cr, fill_color);
+#endif
 
 	if (filled)
 		cairo_fill (cr);
@@ -384,7 +400,11 @@ draw_curve_rectangle (cairo_t * cr,
 /* x, y, width, height are in the xkb coordinate system */
 static void
 draw_rectangle (MatekbdKeyboardDrawingRenderContext * context,
+#if GTK_CHECK_VERSION (3, 0, 0)
+		GdkRGBA * fill_color,
+#else
 		GdkColor * fill_color,
+#endif
 		gint angle,
 		gint xkb_x, gint xkb_y, gint xkb_width, gint xkb_height,
 		gint radius)
@@ -396,7 +416,7 @@ draw_rectangle (MatekbdKeyboardDrawingRenderContext * context,
 		if (fill_color) {
 			filled = TRUE;
 		} else {
-			fill_color = context->dark_color;
+			fill_color = &context->dark_color;
 			filled = FALSE;
 		}
 
@@ -439,7 +459,12 @@ draw_rectangle (MatekbdKeyboardDrawingRenderContext * context,
 static void
 draw_outline (MatekbdKeyboardDrawingRenderContext * context,
 	      XkbOutlineRec * outline,
-	      GdkColor * color, gint angle, gint origin_x, gint origin_y)
+#if GTK_CHECK_VERSION (3, 0, 0)
+	      GdkRGBA * color,
+#else
+	      GdkColor * color,
+#endif
+	      gint angle, gint origin_x, gint origin_y)
 {
 #ifdef KBDRAW_DEBUG
 	printf (" num_points in %p: %d\n", outline, outline->num_points);
@@ -493,55 +518,98 @@ draw_outline (MatekbdKeyboardDrawingRenderContext * context,
 
 /* see PSColorDef in xkbprint */
 static gboolean
+#if GTK_CHECK_VERSION (3, 0, 0)
+parse_xkb_color_spec (gchar * colorspec, GdkRGBA * color)
+#else
 parse_xkb_color_spec (gchar * colorspec, GdkColor * color)
+#endif
 {
 	glong level;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	color->alpha = 1.0;
+#endif
 	if (g_ascii_strcasecmp (colorspec, "black") == 0) {
 		color->red = 0;
 		color->green = 0;
 		color->blue = 0;
 	} else if (g_ascii_strcasecmp (colorspec, "white") == 0) {
-		color->red = 65535;
-		color->green = 65535;
-		color->blue = 65535;
+#if GTK_CHECK_VERSION (3, 0, 0)
+		color->red = 1.0;
+		color->green = 1.0;
+		color->blue = 1.0;
+#else
+		color->red = 0xFFFF;
+		color->green = 0xFFFF;
+		color->blue = 0xFFFF;
+#endif
 	} else if (g_ascii_strncasecmp (colorspec, "grey", 4) == 0 ||
 		   g_ascii_strncasecmp (colorspec, "gray", 4) == 0) {
 		level = strtol (colorspec + 4, NULL, 10);
 
-		color->red = 65535 - 65535 * level / 100;
-		color->green = 65535 - 65535 * level / 100;
-		color->blue = 65535 - 65535 * level / 100;
+#if GTK_CHECK_VERSION (3, 0, 0)
+		color->red = 1.0 - 1.0 * level / 100.0;
+		color->green = 1.0 - 1.0 * level / 100.0;
+		color->blue = 1.0 - 1.0 * level / 100.0;
+#else
+		color->red = 0xFFFF - 0xFFFF * level / 100.0;
+		color->green = 0xFFFF - 0xFFFF * level / 100.0;
+		color->blue = 0xFFFF - 0xFFFF * level / 100.0;
+#endif
 	} else if (g_ascii_strcasecmp (colorspec, "red") == 0) {
-		color->red = 65535;
+#if GTK_CHECK_VERSION (3, 0, 0)
+		color->red = 1.0;
+#else
+		color->red = 0xFFFF;
+#endif
 		color->green = 0;
 		color->blue = 0;
 	} else if (g_ascii_strcasecmp (colorspec, "green") == 0) {
 		color->red = 0;
-		color->green = 65535;
+#if GTK_CHECK_VERSION (3, 0, 0)
+		color->green = 1.0;
+#else
+		color->green = 0xFFFF;
+#endif
 		color->blue = 0;
 	} else if (g_ascii_strcasecmp (colorspec, "blue") == 0) {
 		color->red = 0;
 		color->green = 0;
-		color->blue = 65535;
+#if GTK_CHECK_VERSION (3, 0, 0)
+		color->blue = 1.0;
+#else
+		color->blue = 0xFFFF;
+#endif
 	} else if (g_ascii_strncasecmp (colorspec, "red", 3) == 0) {
 		level = strtol (colorspec + 3, NULL, 10);
 
-		color->red = 65535 * level / 100;
+#if GTK_CHECK_VERSION (3, 0, 0)
+		color->red = 1.0 * level / 100.0;
+#else
+		color->red = 0xFFFF * level / 100.0;
+#endif
 		color->green = 0;
 		color->blue = 0;
 	} else if (g_ascii_strncasecmp (colorspec, "green", 5) == 0) {
 		level = strtol (colorspec + 5, NULL, 10);
 
 		color->red = 0;
-		color->green = 65535 * level / 100;;
+#if GTK_CHECK_VERSION (3, 0, 0)
+		color->green = 1.0 * level / 100.0;
+#else
+		color->green = 0xFFFF * level / 100.0;
+#endif
 		color->blue = 0;
 	} else if (g_ascii_strncasecmp (colorspec, "blue", 4) == 0) {
 		level = strtol (colorspec + 4, NULL, 10);
 
 		color->red = 0;
 		color->green = 0;
-		color->blue = 65535 * level / 100;
+#if GTK_CHECK_VERSION (3, 0, 0)
+		color->blue = 1.0 * level / 100.0;
+#else
+		color->blue = 0xFFFF * level / 100.0;
+#endif
 	} else
 		return FALSE;
 
@@ -808,7 +876,11 @@ draw_pango_layout (MatekbdKeyboardDrawingRenderContext * context,
 		   gint angle, gint x, gint y)
 {
 	PangoLayout *layout = context->layout;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkRGBA *color;
+#else
 	GdkColor *color;
+#endif
 	PangoLayoutLine *line;
 	gint x_off, y_off;
 	gint i;
@@ -856,7 +928,11 @@ draw_pango_layout (MatekbdKeyboardDrawingRenderContext * context,
 	}
 
 	cairo_move_to (context->cr, x, y);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gdk_cairo_set_source_rgba (context->cr, color);
+#else
 	gdk_cairo_set_source_color (context->cr, color);
+#endif
 	pango_cairo_show_layout (context->cr, layout);
 }
 
@@ -1033,7 +1109,13 @@ draw_key (MatekbdKeyboardDrawingRenderContext * context,
 	  MatekbdKeyboardDrawing * drawing, MatekbdKeyboardDrawingKey * key)
 {
 	XkbShapeRec *shape;
-	GdkColor *color;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkStyleContext *style_context;
+	GdkRGBA color;
+#else
+	GtkStyle *style;
+	GdkColor color;
+#endif
 	XkbOutlineRec *outline;
 	int origin_offset_x;
 	/* gint i; */
@@ -1049,23 +1131,32 @@ draw_key (MatekbdKeyboardDrawingRenderContext * context,
 
 	shape = drawing->xkb->geom->shapes + key->xkbkey->shape_ndx;
 
-	if (key->pressed)
-		color =
-		    &gtk_widget_get_style (GTK_WIDGET (drawing))->base
-		    [GTK_STATE_SELECTED];
-	else
-		color = drawing->colors + key->xkbkey->color_ndx;
+	if (key->pressed) {
+#if GTK_CHECK_VERSION (3, 0, 0)
+		style_context = gtk_widget_get_style_context (GTK_WIDGET (drawing));
+		gtk_style_context_save (style_context);
+		gtk_style_context_add_class (style_context, GTK_STYLE_CLASS_VIEW);
+		gtk_style_context_get_background_color (style_context,
+		                                        GTK_STATE_FLAG_SELECTED,
+		                                        &color);
+		gtk_style_context_restore (style_context);
+#else
+		style = gtk_widget_get_style (GTK_WIDGET (drawing));
+		color = style->base[GTK_STATE_SELECTED];
+#endif
+	 } else
+		color = *(drawing->colors + key->xkbkey->color_ndx);
 
 #ifdef KBDRAW_DEBUG
 	printf
-	    (" outlines base in the shape: %p (total: %d), origin: (%d, %d), angle %d, colored: %s\n",
+	    (" outlines base in the shape: %p (total: %d), origin: (%d, %d), angle %d\n",
 	     shape->outlines, shape->num_outlines, key->origin_x,
-	     key->origin_y, key->angle, color ? "yes" : "no");
+	     key->origin_y, key->angle);
 #endif
 
 	/* draw the primary outline */
 	outline = shape->primary ? shape->primary : shape->outlines;
-	draw_outline (context, outline, color, key->angle, key->origin_x,
+	draw_outline (context, outline, &color, key->angle, key->origin_x,
 		      key->origin_y);
 #if 0
 	/* don't draw other outlines for now, since
@@ -1195,7 +1286,11 @@ draw_indicator_doodad (MatekbdKeyboardDrawingRenderContext * context,
 		       MatekbdKeyboardDrawingDoodad * doodad,
 		       XkbIndicatorDoodadRec * indicator_doodad)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkRGBA *color;
+#else
 	GdkColor *color;
+#endif
 	XkbShapeRec *shape;
 	gint i;
 
@@ -1222,7 +1317,11 @@ draw_shape_doodad (MatekbdKeyboardDrawingRenderContext * context,
 		   XkbShapeDoodadRec * shape_doodad)
 {
 	XkbShapeRec *shape;
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GdkRGBA *color;
+#else
 	GdkColor *color;
+#endif
 	gint i;
 
 	if (!drawing->xkb)
@@ -1350,7 +1449,16 @@ draw_keyboard_to_context (MatekbdKeyboardDrawingRenderContext * context,
 static gboolean
 create_cairo (MatekbdKeyboardDrawing * drawing)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkStyleContext *style_context = NULL;
+	GtkStateFlags state;
+	GdkRGBA dark_color;
+#else
+	GtkStyle *style = NULL;
 	GtkStateType state;
+	GdkColor dark_color;
+#endif
+
 	if (drawing == NULL)
 		return FALSE;
 	if (drawing->surface == NULL)
@@ -1359,9 +1467,25 @@ create_cairo (MatekbdKeyboardDrawing * drawing)
 	drawing->renderContext->cr =
 	    cairo_create (drawing->surface);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	style_context = gtk_widget_get_style_context (GTK_WIDGET (drawing));
+	state = gtk_style_context_get_state (style_context);
+
+	gtk_style_context_get_background_color (style_context, state,
+	                                        &dark_color);
+	/* make dark background by making regular background darker */
+	dark_color.red *= 0.7;
+	dark_color.green *= 0.7;
+	dark_color.blue *= 0.7;
+#else
+	style = gtk_widget_get_style (GTK_WIDGET (drawing));
 	state = gtk_widget_get_state (GTK_WIDGET (drawing));
-	drawing->renderContext->dark_color =
-	    &gtk_widget_get_style (GTK_WIDGET (drawing))->dark[state];
+
+	dark_color = style->dark[state];
+#endif
+
+	drawing->renderContext->dark_color = dark_color;
+
 	return TRUE;
 }
 
@@ -1370,14 +1494,20 @@ destroy_cairo (MatekbdKeyboardDrawing * drawing)
 {
 	cairo_destroy (drawing->renderContext->cr);
 	drawing->renderContext->cr = NULL;
-	drawing->renderContext->dark_color = NULL;
 }
 
 static void
 draw_keyboard (MatekbdKeyboardDrawing * drawing)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+        GtkStyleContext *context =
+	    gtk_widget_get_style_context (GTK_WIDGET (drawing));
+	GtkStateFlags state = gtk_style_context_get_state (context);
+	GdkRGBA color;
+#else
+	GtkStyle *style = gtk_widget_get_style (GTK_WIDGET (drawing));
 	GtkStateType state = gtk_widget_get_state (GTK_WIDGET (drawing));
-        GtkStyle *style = gtk_widget_get_style (GTK_WIDGET (drawing));
+#endif
 	GtkAllocation allocation;
 
 	if (!drawing->xkb)
@@ -1393,10 +1523,18 @@ draw_keyboard (MatekbdKeyboardDrawing * drawing)
 					       allocation.height);
 
 	if (create_cairo (drawing)) {
-	        /* blank background */
-                gdk_cairo_set_source_color (drawing->renderContext->cr,
-                                            &style->base[state]);
-                cairo_paint (drawing->renderContext->cr);
+		/* blank background */
+#if GTK_CHECK_VERSION (3, 0, 0)
+		gtk_style_context_save (context);
+		gtk_style_context_add_class (context, GTK_STYLE_CLASS_VIEW);
+		gtk_style_context_get_background_color (context, state, &color);
+		gtk_style_context_restore (context);
+		gdk_cairo_set_source_rgba (drawing->renderContext->cr, &color);
+#else
+		gdk_cairo_set_source_color (drawing->renderContext->cr,
+		                            &style->base[state]);
+#endif
+		cairo_paint (drawing->renderContext->cr);
 
 		draw_keyboard_to_context (drawing->renderContext, drawing);
 		destroy_cairo (drawing);
@@ -1412,13 +1550,24 @@ alloc_render_context (MatekbdKeyboardDrawing * drawing)
 
 	PangoContext *pangoContext =
 	    gtk_widget_get_pango_context (GTK_WIDGET (drawing));
+
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkStyleContext *style_context =
+	    gtk_widget_get_style_context (GTK_WIDGET (drawing));
+	PangoFontDescription *fd = NULL;
+
+	gtk_style_context_get (style_context,
+	                       gtk_style_context_get_state (style_context),
+	                       GTK_STYLE_PROPERTY_FONT, &fd, NULL);
+#else
+	GtkStyle *style = gtk_widget_get_style (GTK_WIDGET (drawing));
+	PangoFontDescription *fd = style->font_desc;
+#endif
+
 	context->layout = pango_layout_new (pangoContext);
 	pango_layout_set_ellipsize (context->layout, PANGO_ELLIPSIZE_END);
 
-	context->font_desc =
-	    pango_font_description_copy (gtk_widget_get_style
-					 (GTK_WIDGET
-					  (drawing))->font_desc);
+	context->font_desc = pango_font_description_copy (fd);
 	context->angle = 0;
 	context->scale_numerator = 1;
 	context->scale_denominator = 1;
@@ -1844,7 +1993,11 @@ init_colors (MatekbdKeyboardDrawing * drawing)
 	if (!drawing->xkb)
 		return;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+	drawing->colors = g_new (GdkRGBA, drawing->xkb->geom->num_colors);
+#else
 	drawing->colors = g_new (GdkColor, drawing->xkb->geom->num_colors);
+#endif
 
 	for (i = 0; i < drawing->xkb->geom->num_colors; i++) {
 		result =
@@ -2237,26 +2390,50 @@ matekbd_keyboard_drawing_render (MatekbdKeyboardDrawing * kbdrawing,
 			      double width, double height,
 			      double dpi_x, double dpi_y)
 {
+#if GTK_CHECK_VERSION (3, 0, 0)
+	GtkStyleContext *style_context =
+	    gtk_widget_get_style_context (GTK_WIDGET (kbdrawing));
+	GdkRGBA dark_color;
+	PangoFontDescription *fd = NULL;
+
+	gtk_style_context_get_background_color (style_context,
+	                                        gtk_style_context_get_state (style_context),
+	                                        &dark_color);
+	/* make dark background by making regular background darker */
+	dark_color.red *= 0.7;
+	dark_color.green *= 0.7;
+	dark_color.blue *= 0.7;
+
+	gtk_style_context_get (style_context,
+	                       gtk_style_context_get_state (style_context),
+	                       GTK_STYLE_PROPERTY_FONT, &fd, NULL);
+	fd = pango_font_description_copy (fd);
+#else
+	GtkStyle *style = gtk_widget_get_style (GTK_WIDGET (kbdrawing));
+	GdkColor dark_color;
+	PangoFontDescription *fd =
+	    pango_font_description_copy (style->font_desc);
+
+	dark_color = style->dark[gtk_widget_get_state (GTK_WIDGET (kbdrawing))];
+#endif
+
 	MatekbdKeyboardDrawingRenderContext context = {
 		cr,
 		kbdrawing->renderContext->angle,
 		layout,
-		pango_font_description_copy (gtk_widget_get_style
-					     (GTK_WIDGET
-					      (kbdrawing))->font_desc),
+		pango_font_description_copy (fd),
 		1, 1,
-		&gtk_widget_get_style (GTK_WIDGET (kbdrawing))->dark
-		    [gtk_widget_get_state (GTK_WIDGET (kbdrawing))]
+		dark_color
 	};
 
 	if (!context_setup_scaling (&context, kbdrawing, width, height,
-				    dpi_x, dpi_y))
+	                            dpi_x, dpi_y))
 		return FALSE;
 	cairo_translate (cr, x, y);
 
 	draw_keyboard_to_context (&context, kbdrawing);
 
-	pango_font_description_free (context.font_desc);
+	pango_font_description_free (fd);
 
 	return TRUE;
 }
