@@ -36,6 +36,10 @@
 
 #include "X11/XKBlib.h"
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+#define gtk_vbox_new(X,Y) gtk_box_new(GTK_ORIENTATION_VERTICAL,Y)
+#endif
+
 int main(int argc, char** argv)
 {
 	GtkWidget* gki;
